@@ -159,7 +159,7 @@ class ArchitectureX86(Architecture):
             (b'\xff[\x90\x91\x92\x93\x94\x96\x97][\x00-\x0ff]{4}', 6)]
 
     def _initBadInstructions(self):
-        self._badInstructions = ['retf','enter','loop','loopne','int3', 'db', 'ret', 'call', 'jmp']
+        self._badInstructions = ['retf','enter','loop','loopne','int3', 'db']
 
     def _initCategories(self):
         self._categories = {
