@@ -21,7 +21,7 @@ from ropper.common.utils import *
 from ropper.common.error import *
 from ropper.common.enum import Enum
 from ropper.arch import x86
-from multiprocessing import Process, Pool, Queue, cpu_count, current_process
+from multiprocessing import Process, Pool, Queue, cpu_count, current_process, JoinableQueue
 from .gadget import Gadget, GadgetType
 from binascii import hexlify, unhexlify
 from struct import pack
@@ -37,7 +37,7 @@ except:
     pass
 
 
-class FORMAT(Enum):
+class Format(Enum):
     _enum_ = 'RAW STRING HEX'
 
 class Ropper(object):
@@ -57,13 +57,13 @@ class Ropper(object):
             self.__cs = capstone.Cs(arch.arch, arch.mode)
         return self.__cs
 
-    def assemble(self, code, arch=x86, format=FORMAT.HEX):
+    def assemble(self, code, arch=x86, format=Format.HEX):
         if 'keystone' not in globals():
             raise RopperError('Keystone is not installed! Please install Keystone. \nLook at http://keystone-engine.org')
 
         ks = keystone.Ks(arch.ksarch[0], arch.ksarch[1])
         try:
-            byte_list =  ks.asm(code)[0]
+            byte_list =  ks.asm(code.encode('ascii'))[0]
         except BaseException as e:
             raise RopperError(e)
 
@@ -71,17 +71,17 @@ class Ropper(object):
             return "invalid"
         to_return = byte_list
 
-        if format == FORMAT.STRING:
+        if format == Format.STRING:
             to_return = '"'
             for byte in byte_list:
                 to_return += '\\x%02x' % byte
 
             to_return += '"'
-        elif format == FORMAT.HEX:
+        elif format == Format.HEX:
             to_return = ''
             for byte in byte_list:
                 to_return += '%02x' % byte
-        elif format == FORMAT.RAW:
+        elif format == Format.RAW:
             to_return = ''
             for byte in byte_list:
                 to_return += '%s' % chr(byte)
@@ -112,6 +112,7 @@ class Ropper(object):
 
     def searchJmpReg(self, binary, regs):
         toReturn = []
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         for section in binary.executableSections:
 
             gadgets = self._searchJmpReg(section, binary, regs)
@@ -124,6 +125,7 @@ class Ropper(object):
         if binary.arch.arch != capstone.CS_ARCH_X86:
             raise NotSupportedError(
                 'Wrong architecture, \'jmp <reg>\' only supported on x86/x86_64')
+
         cs = self.__getCs(binary.arch)
         toReturn = []
         Register = Enum('Register', 'ax cx dx bx sp bp si di')
@@ -135,7 +137,7 @@ class Ropper(object):
             insts = [toBytes(0xff , 0xe0 | Register[reg_tmp]), toBytes(0xff, 0xd0 | Register[reg_tmp]),  toBytes(0x50 | Register[reg_tmp] , 0xc3)]
 
             for inst in insts:
-                toReturn.extend(self._searchOpcode(section, binary, inst, True))
+                toReturn.extend(self._searchOpcode(section, binary, inst, len(inst),True))
 
         return sorted(toReturn, key=lambda x: str(x))
 
@@ -146,7 +148,7 @@ class Ropper(object):
             raise RopperError('The length of the opcode has to be a multiple of two')
 
         opcode = opcode.encode('ascii')
-        size = len(opcode)/2
+        size = int(len(opcode)/2)
         for b in (b'5c',b'5d',b'5b',b'28',b'29',b'2b',b'2a',b'2e',b'3f'):
            
             if opcode.find(b) % 2 == 0:
@@ -154,9 +156,11 @@ class Ropper(object):
 
         m = re.search(b'\?', opcode)
         while m:
-
             if m.start() % 2 == 0:
-                if opcode[m.start()+1] == '?':
+                char = opcode[m.start()+1]
+                if type(char) == int:
+                    char = chr(char)
+                if char == '?':
                     opcode = opcode[:m.start()] + hexlify(b'[\x00-\xff]') +  opcode[m.start()+2:]
                 else:
                     raise RopperError('A ? for the highest 4 bit of a byte is not supported (e.g. ?1, ?2, ..., ?a)')
@@ -182,12 +186,13 @@ class Ropper(object):
 
 
     def searchInstructions(self, binary, code):
-
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         opcode = self.assemble(code, binary.arch)
         return self.searchOpcode(binary, opcode, disass=True)
 
 
     def searchOpcode(self, binary, opcode, disass=False):
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         opcode, size = self._formatOpcodeString(opcode)
         gadgets = []
         for section in binary.executableSections:
@@ -203,14 +208,18 @@ class Ropper(object):
         code = bytearray(section.bytes)
         offset = section.offset
         for match in re.finditer(opcode, code):
-            opcodeGadget = Gadget(binary, section)
+            opcodeGadget = Gadget(binary.fileName, section.name, binary.arch)
 
             if (offset + match.start()) % binary.arch.align == 0:
                 if disass:
+                    could_disass = False
                     #for i in disassembler.disasm(struct.pack('B' * size, *code[match.start():match.end()]), offset + match.start()):
                     for i in disassembler.disasm(struct.pack('B' * size, *code[match.start():match.end()]), offset + match.start()):
                         opcodeGadget.append(
                             i.address, i.mnemonic , i.op_str, bytes=i.bytes)
+                        could_disass = True
+                    if not could_disass:
+                        continue
                 else:
                     opcodeGadget.append(
                         offset + match.start(), hexlify(match.group(0)).decode('utf-8'),bytes=match.group())
@@ -223,6 +232,7 @@ class Ropper(object):
 
 
     def searchPopPopRet(self, binary):
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         toReturn = []
         for section in binary.executableSections:
 
@@ -241,24 +251,19 @@ class Ropper(object):
         code = section.bytes
         offset = section.offset
         toReturn = []
-
-        for index in range(len(code)):
-            if code[index] == 0xc3 and 0 not in code[index - 2:index + 1]:
-                ppr = Gadget(binary,section)
-                for i in disassembler.disasm(struct.pack('BBB', *code[index - 2:index + 1]), offset + index -2):
-                    if i.mnemonic != 'pop' and i.mnemonic != 'ret':
-                        break
-                    ppr.append(
-                        i.address, i.mnemonic , i.op_str, bytes=i.bytes)
-
-                    if i.mnemonic.startswith('ret'):
-                        break
-                if len(ppr) == 3:
-
-                    toReturn.append(ppr)
+        pprs = binary.arch.pprs
+        for ppr in pprs:
+            for match in re.finditer(ppr, code):
+                if (offset + match.start()) % binary.arch.align == 0:
+                    pprg = Gadget(binary.fileName,section.name, binary.arch)
+                    for i in disassembler.disasm(bytes(bytearray(code)[match.start():match.end()]), offset + match.start()):
+                        pprg.append(i.address, i.mnemonic , i.op_str, bytes=i.bytes)
+        
+                    toReturn.append(pprg)
         return toReturn
 
     def searchGadgets(self, binary, instructionCount=5, gtype=GadgetType.ALL):
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         gadgets = []
         for section in binary.executableSections:
             vaddr = binary.imageBase
@@ -271,9 +276,6 @@ class Ropper(object):
             else:
                 newGadgets = self._searchGadgetsForked(section=section, binary=binary, instruction_count=instructionCount, gtype=gtype)
             
-                for gadget in newGadgets:
-                    gadget.binary = binary
-                    gadget.section = section
             gadgets.extend(newGadgets)
 
         return sorted(gadgets, key=Gadget.simpleInstructionString)
@@ -288,7 +290,7 @@ class Ropper(object):
 
         max_progress = len(code) * len(arch.endings[gtype])
 
-        vaddrs = set() # to prevent that the gadget is added several times
+        vaddrs = set()
         for ending in arch.endings[gtype]:
             offset_tmp = 0
             tmp_code = code[:]
@@ -304,7 +306,7 @@ class Ropper(object):
 
                     for x in range(0, index, arch.align):
                         code_part = tmp_code[index - x-1:index + ending[1]]
-                        gadget, leng = self.__createGadget(arch, code_part, offset + offset_tmp - x, ending,binary, section)
+                        gadget, leng = self.__createGadget(arch, code_part, offset + offset_tmp - x, ending,binary.fileName, section.name)
                         if gadget:
                             if leng > instruction_count:
                                 break
@@ -315,7 +317,7 @@ class Ropper(object):
                             none_count = 0
                         else:
                             none_count += 1
-                            if none_count == 5:
+                            if none_count == arch.maxInvalid:
                                 break
 
                 tmp_code = tmp_code[index+arch.align:]
@@ -342,21 +344,24 @@ class Ropper(object):
 
         max_progress = len(code) * len(arch.endings[gtype])
 
-        ending_queue = Queue()
+        ending_queue = JoinableQueue()
         gadget_queue = Queue()
         tmp_code = code[:]
 
         process_count = min(cpu_count()+1, len(arch.endings[gtype]))
-
         for ending in arch.endings[gtype]:
             ending_queue.put(ending)
 
         for cpu in range(process_count):
-            processes.append(Process(target=self.__gatherGadgetsByEndings, args=(section, binary, tmp_code, arch, ending_queue, gadget_queue, instruction_count), name="GadgetSearch%d"%cpu))
+            ending_queue.put(None)
+
+        for cpu in range(process_count):
+            processes.append(Process(target=self.__gatherGadgetsByEndings, args=(tmp_code, arch, binary.fileName, section.name, section.offset, ending_queue, gadget_queue, instruction_count), name="GadgetSearch%d"%cpu))
             processes[cpu].daemon=True
             processes[cpu].start()
 
         
+        
         count = 0
         ending_count = 0
         if self.__callback:
@@ -369,37 +374,32 @@ class Ropper(object):
                 ending_count += 1
                 if self.__callback:
                     self.__callback(section, to_return, float(ending_count) / len(arch.endings[gtype]))
-            #else:
-             #   count +=1
-        
-        for process in processes:
-            process.terminate()
             
         return to_return
 
-    def __gatherGadgetsByEndings(self, section, binary, code, arch, ending_queue, gadget_queue, instruction_count):
+    def __gatherGadgetsByEndings(self,code, arch, fileName, sectionName, offset, ending_queue, gadget_queue, instruction_count):
         
-        try:
-            while not ending_queue.empty():
-                try:
-                    ending = ending_queue.get(False)
-                    gadgets = self.__gatherGadgetsByEnding(code, arch, ending, instruction_count)
-                    
-                    gadget_queue.put(gadgets)
-                except:
-                    pass
+        #try:
+        while True:
+            ending = ending_queue.get()
+            if ending is None:
+                ending_queue.task_done()
+                break
             
-        except BaseException as e:
-            raise RopperError(e)
-        
-        gadget_queue.put(None)
+            gadgets = self.__gatherGadgetsByEnding(code, arch, fileName, sectionName, offset, ending, instruction_count)
+            
+            gadget_queue.put(gadgets)
+            ending_queue.task_done()
+            
+            
+        #except BaseException as e:
+        #    raise RopperError(e)
         
 
-    def __gatherGadgetsByEnding(self, code, arch, ending, instruction_count):
-        vaddrs = set() # to prevent that the gadget is added several times
+    def __gatherGadgetsByEnding(self, code, arch, fileName, sectionName, offset, ending, instruction_count):
+        vaddrs = set()
         offset_tmp = 0
-        #offset = section.offset
-        offset = 0
+        
         tmp_code = code[:]
         to_return = []
         match = re.search(ending[0], tmp_code)
@@ -414,18 +414,16 @@ class Ropper(object):
 
                 for x in range(0, index+1, arch.align):
                     code_part = tmp_code[index - x:index + ending[1]]
-                    gadget, leng = self.__createGadget(arch, code_part, offset + offset_tmp - x , ending)
+                    gadget, leng = self.__createGadget(arch, code_part, offset + offset_tmp - x , ending, fileName, sectionName)
                     if gadget:
                         if leng > instruction_count:
                             break
                         if gadget:
-                            #if gadget.address not in vaddrs:
-                            #    vaddrs.update([gadget.address])
                             to_return.append(gadget)
                         none_count = 0
                     else:
                         none_count += 1
-                        if none_count == 5:
+                        if none_count == arch.maxInvalid:
                             break
 
             tmp_code = tmp_code[index+arch.align:]
@@ -433,15 +431,10 @@ class Ropper(object):
 
             match = re.search(ending[0], tmp_code)
 
-            #if self.__callback:
-            #    progress = arch.endings[gtype].index(ending) * len(code) + len(code) - len(tmp_code)
-            #    self.__callback(section, toReturn, float(progress) / max_progress)
         return to_return
-        #if self.__callback:
-        #    self.__callback(section, toReturn, 1.0)
 
     def __createGadget(self, arch, code_str, codeStartAddress, ending, binary=None, section=None):
-        gadget = Gadget(binary, section)
+        gadget = Gadget(binary, section, arch)
         hasret = False
 
         disassembler = self.__getCs(arch)
@@ -466,14 +459,14 @@ class Ropper(object):
 
 
     def __disassembleBackward(self, section, binary, vaddr,offset, count):
-        gadget = Gadget(binary, section)
+        gadget = Gadget(binary.fileName, section.name, binary.arch)
         counter = 0
         toReturn = None
         code = bytes(bytearray(section.bytes))
         disassembler = self.__getCs(binary.arch)
 
         while len(gadget) < count:
-            gadget = Gadget(binary, section)
+            gadget = Gadget(binary.fileName, section.name, binary.arch)
             for i in disassembler.disasm(struct.pack('B' * len(code[offset - counter:]), *bytearray(code[offset - counter:])), vaddr-counter):
                 gadget.append(i.address, i.mnemonic , i.op_str, i.bytes)
                 if i.address == vaddr:
@@ -482,7 +475,7 @@ class Ropper(object):
                 if i.address > vaddr:
                     if len(gadget) > count:
                         return toReturn
-                    gadget = Gadget(binary, section)
+                    gadget = Gadget(binary.fileName, section.name, binary.arch)
                     break
 
 
@@ -491,7 +484,7 @@ class Ropper(object):
                 return toReturn
 
             if not toReturn:
-                toReturn = Gadget(binary, section)
+                toReturn = Gadget(binary.fileName, section.name, binary.arch)
                 toReturn.append(vaddr,'bad instructions')
         return toReturn
 
@@ -499,13 +492,13 @@ class Ropper(object):
     def disassembleAddress(self, section, binary, vaddr, offset, count):
         if vaddr % binary.arch.align != 0:
             raise RopperError('The address doesn\'t have the correct alignment')
-
+        Gadget.IMAGE_BASES[binary.fileName] = binary.imageBase
         code = bytes(bytearray(section.bytes))
         disassembler = capstone.Cs(binary.arch.arch, binary.arch.mode)
 
         if count < 0:
             return self.__disassembleBackward(section, binary, vaddr, offset, count*-1)
-        gadget  = Gadget(binary, section)
+        gadget  = Gadget(binary.fileName, section.name, binary.arch)
         c = 0
 
         for i in disassembler.disasm(struct.pack('B' * len(code[offset:]), *bytearray(code[offset:])), offset):
